unit CodeCompletionSettingsU;

interface

uses
  Registry,
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ColorGrd, StdCtrls;

type
  TMainForm = class(TForm)
    grdColour: TColorGrid;
    btnApply: TButton;
    btnClose: TButton;
    lstElement: TListBox;
    Label1: TLabel;
    Label2: TLabel;
    chkAutoParens: TCheckBox;
    Label3: TLabel;
    memExtraUnits: TMemo;
    btnDefaultExtraUnits: TButton;
    procedure FormCreate(Sender: TObject);
    procedure lstElementClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnApplyClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure grdColourChange(Sender: TObject);
    procedure btnDefaultExtraUnitsClick(Sender: TObject);
  private
    { Private declarations }
  public
    Reg: TRegIniFile;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

type
  TColourSetting = record
    Desc, Ident, Default, NewColour: String;
  end;

var
  ColourSettings: array[0..8] of TColourSetting =
  ((Desc: 'Property'; Ident:'PropSymbolColor'; Default: 'clNavy'),
   (Desc: 'Variable'; Ident:'VarSymbolColor'; Default: 'clMaroon'),
   (Desc: 'Constant'; Ident:'ConstSymbolColor'; Default: 'clGreen'),
   (Desc: 'Type'; Ident:'TypeSymbolColor'; Default: 'clOlive'),
   (Desc: 'Procedure'; Ident:'ProcSymbolColor'; Default: 'clTeal'),
   (Desc: 'Function'; Ident:'FuncSymbolColor'; Default: 'clBlue'),
   (Desc: 'Unit'; Ident:'UnitSymbolColor'; Default: 'clBlack'),
   (Desc: 'Label'; Ident:'LabelSymbolColor'; Default: 'clTeal'),
   (Desc: 'Abstract method'; Ident: 'AbstractSymbolColor'; Default: 'clRed'));

const
  RegPath = 'Software\Borland\Delphi\6.0';
  RegSection = 'Code Insight';
  RegAutoParensSetting = 'CodeCompleteAutoParens';
  RegExtraUnitsSetting = 'ExtraUnits';
  DefaultExtraUnits = 'Classes;ComConst;ComObj;ComServ;Contnrs;ConvUtils;DateUtils;HelpIntfs;IniFiles;MaskUtils;Mask;Math;MtsObj;RTLConsts;Registry;StdConvs;StdVCL;StrUtils;SyncObjs;TypInfo;VCLCom;VarCmplx;VarConv;ShareMem;SysConst;SysUtils;Types;VarUtils;Variants';

procedure TMainForm.FormCreate(Sender: TObject);
var
  I: Integer;
  S: String;
begin
  Reg := TRegIniFile.Create(RegPath);
  //Read auto-parentheses setting and update checkbox
  chkAutoParens.Checked := Reg.ReadBool(RegSection,
    RegAutoParensSetting, True);
  //Set up listbox
  for I := Low(ColourSettings) to High(ColourSettings) do
    lstElement.Items.Add(ColourSettings[I].Desc);
  //Select first item
  lstElement.ItemIndex := 0;
  //and select the corresponding colour without the grid event firing
  //(event handler is disconnected)
  lstElementClick(lstElement);
  //Now connect up the grid event handler to react to user selections
  grdColour.OnChange := grdColourChange;
  //Read the ExtraUnits value, separating units into separate lines
  S := Reg.ReadString(RegSection, RegExtraUnitsSetting, '');
  memExtraUnits.Lines.Text := StringReplace(S, ';', #13#10, [rfReplaceAll])
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  Reg.Free;
end;

procedure TMainForm.lstElementClick(Sender: TObject);
var
  CurrentColourStr: String;
  CurrentColour: TColor;
begin
  //Get selected colour item
  with ColourSettings[lstElement.ItemIndex] do
  begin
    //Try and find a new colour that hasn't been applied yet
    CurrentColourStr := NewColour;
    //If not, try and get a colour from the registry, else use default
    if CurrentColourStr = '' then
      CurrentColourStr := Reg.ReadString(RegSection, Ident, Default);
  end;
  //Set colour on colour grid
  CurrentColour := StringToColor(CurrentColourStr);
  grdColour.ForegroundIndex := grdColour.ColorToIndex(CurrentColour)
end;

procedure TMainForm.btnApplyClick(Sender: TObject);
var
  I: Integer;
  ExtraUnits: String;
begin
  //Write all changed colours to the registry
  for I := Low(ColourSettings) to High(ColourSettings) do
    with ColourSettings[I] do
      if NewColour <> '' then
      begin
        Reg.WriteString(RegSection, Ident, NewColour);
        NewColour := '';
      end;
  //Write auto-parentheses setting to the registry
  Reg.WriteBool(RegSection, RegAutoParensSetting,
    chkAutoParens.Checked);
  //Write Extra Units to registry if specified, separated by semicolons
  ExtraUnits := StringReplace(
    Trim(memExtraUnits.Lines.Text), #13#10, ';', [rfReplaceAll]);
  if ExtraUnits <> '' then
    Reg.WriteString(RegSection, RegExtraUnitsSetting, ExtraUnits);
end;

procedure TMainForm.btnCloseClick(Sender: TObject);
begin
  Close
end;

procedure TMainForm.grdColourChange(Sender: TObject);
begin
  //When user clicks on new colour, store it in array
  ColourSettings[lstElement.ItemIndex].NewColour :=
    ColorToString(grdColour.ForegroundColor)
end;

procedure TMainForm.btnDefaultExtraUnitsClick(Sender: TObject);
begin
  memExtraUnits.Lines.Text :=
    StringReplace(DefaultExtraUnits, ';', #13#10, [rfReplaceAll])
end;

end.
